#include <Arduino.h>
#include "Encoder.h"
#include "PID_v1.h"
#include "SimpleSerialization.h"
#include "EnginesControllerMessage.h"
#include "KvFloatMessage.h"

#define INPUT_OFFSET  0
#define INPUT_REQUEST  1
#define OUTPUT_TOTAL  2
#define REACHED_POINT  3
#define INPUT_DELAYS 4
/*
// Motor A

int enA = 9;
int in1 = 8;
int in2 = 7;

// Motor B

int enB = 12;
int in3 = 11;
int in4 = 10;
*/
// Motor A

int enA = 12;
int in1 = 11;
int in2 = 10;

// Motor B

int enB = 9;
int in3 = 8;
int in4 = 7;

//counters
Encoder lf(2,3);
Encoder rf(18,19);
Encoder lr(20,22);
Encoder rr(21,23);

double pl = 1;
double il = 0;
double dl = 0;

double pr = 1;
double ir = 0;
double dr = 0;

double requestLeft = 0;
double requestRight = 0;
double leftInput = 0;
double leftOutput = 0;
double rightInput = 0;
double rightOutput = 0;

long leftTotal = 0;
long rightTotal = 0;
long leftOffset = 0;
long rightOffset = 0;
bool offsetEnable = false;

long encoderDelay=50;
long sendDelay=300;
unsigned long lastEncoder=0;
unsigned long lastSend=0;
bool debugTicks = false;
bool frontcalcOnly = false;

PID left(&leftInput, &leftOutput, &requestLeft, pl,il,dl, DIRECT);
PID right(&rightInput, &rightOutput, &requestRight, pr,ir,dr, DIRECT);

EnginesControllerMessage ecm;
EnginesControllerMessage output;
KvFloatMessage pidMessage;

bool compare(long total, long offset){
  if(offset==0) return true;
  if(offset > 0){
    if(total > offset) return true;
    return false;
  }
  if(offset < 0){
    if(total < offset) return true;
    return false;
  }
  return false;
}

void setMotors(double left,double right){
  if(ecm.getEnable()!=1){
    analogWrite(enA, 0);
    analogWrite(enB, 0);
    return;
  }
  if(left>0){
    digitalWrite(in1,HIGH);
    digitalWrite(in2,LOW);
  } 
  if(left<0){
    digitalWrite(in1,LOW);
    digitalWrite(in2,HIGH);
  }
  if(right>0){
    digitalWrite(in3,HIGH);
    digitalWrite(in4,LOW);
  }
  if (right<0){
    digitalWrite(in3,LOW);
    digitalWrite(in4,HIGH);
  }
  analogWrite(enA, (int)abs(left));
  analogWrite(enB, (int)abs(right));
}

void stopMotors(){
  ecm.setEnable(0);
  setMotors(0,0);
}

void processPid(SerializationData *data){
  switch(pidMessage.getKey()){
    case 0:
      pl = pr = pidMessage.getValue();
      break;
      case 1:
      il = ir = pidMessage.getValue();
      break;
      case 2:
      dl = dr = pidMessage.getValue();
      break;
      case 3:
      debugTicks = ((int)pidMessage.getValue())==1;
      break;
      frontcalcOnly = ((int)pidMessage.getValue()==1);
      default:
      break;
  }
}

void processInput(SerializationData *data){
  switch (ecm.getMode())
  {
    case INPUT_OFFSET:
      leftOffset = ecm.getLeft();
      rightOffset = ecm.getRight();
      offsetEnable = true;
      break;
    case INPUT_REQUEST:
      requestLeft = ecm.getLeft();
      requestRight = ecm.getRight();
      break;
      case INPUT_DELAYS:
      encoderDelay = ecm.getLeft();
      sendDelay = ecm.getRight();
    default:
      break;
  }
}

long calcTicks(long front, long rear){
  if(frontcalcOnly){
    return front;
  }else{
    return (front+rear)/2;
  }
}

void processEncoders(){
  if(debugTicks){
    long lfc = lf.readAndReset()*-1;
    long lrc = lr.readAndReset()*-1;

    long rfc = rf.readAndReset();
    long rrc = rr.readAndReset();

    output.setLeft(lfc);
    output.setRight(rfc);
    output.setMode(1);
    SimpleSerialization.write(&output);

    output.setLeft(lrc);
    output.setRight(rrc);
    output.setMode(2);
    SimpleSerialization.write(&output);
    
    leftInput = calcTicks(lfc,lrc);
    rightInput = calcTicks(rfc,rrc);

  }else{
    leftInput = calcTicks(lf.readAndReset()*-1,lr.readAndReset()*-1);
    rightInput = calcTicks(rf.readAndReset(),rr.readAndReset());
  }
  leftTotal += leftInput;
  rightTotal += rightInput;
  left.Compute();
  right.Compute();
  //output.setLeft(leftOutput);
  //output.setRight(rightOutput);
  //output.setMode(32);
  //SimpleSerialization.write(&output);
  setMotors(leftOutput,rightOutput);
  //setMotors(leftInput,rightInput);
}

void setup() {
  Serial.begin(115200);
  pinMode(enA, OUTPUT);
  pinMode(enB, OUTPUT);
  pinMode(in1, OUTPUT);
  pinMode(in2, OUTPUT);
  pinMode(in3, OUTPUT);
  pinMode(in4, OUTPUT);
  ecm.setPreamble(4294901761);
  ecm.setPostamble(4294901762);
  output.setPreamble(4294901763);
  output.setPostamble(4294901764);
  pidMessage.setPreamble(4294901765);
  pidMessage.setPostamble(4294901766);
  SimpleSerialization.begin(&Serial2,115200);
  left.SetOutputLimits(-255.0,255.0);
  right.SetOutputLimits(-255.0,255.0);
  ecm.setUpdateCallback(&processInput);
  pidMessage.setUpdateCallback(&processPid);
  SimpleSerialization.addDeserializableData(&ecm);
  SimpleSerialization.addDeserializableData(&pidMessage);
  left.SetMode(AUTOMATIC);
  right.SetMode(AUTOMATIC);
  digitalWrite(13,HIGH);
  delay(100);
  digitalWrite(13,LOW);
  delay(100);
  digitalWrite(13,HIGH);
  delay(100);
  digitalWrite(13,LOW);
}

void loop() {
  while(Serial2.available()>0){
    SimpleSerialization.processInput();
  }
  
  if(lastEncoder+encoderDelay<=millis()){
    lastEncoder = millis();
    processEncoders();
    if(offsetEnable){
      bool leftReached = compare(leftTotal, leftOffset);
      bool rightReached = compare(rightTotal, rightOffset);
      if(leftReached) requestLeft = 0;
      if(rightReached) requestRight = 0;
      if(leftReached && rightReached){
        stopMotors();
        output.setLeft(leftTotal);
        output.setRight(rightTotal);
        output.setMode(0);
        SimpleSerialization.write(&output);
        leftTotal = 0;
        rightTotal = 0;
        offsetEnable = false;
      }
    }
  }
  
  if(lastSend+sendDelay<=millis()){
    lastSend = millis();
    output.setLeft(leftTotal);
    output.setRight(rightTotal);
    output.setMode(3);
    SimpleSerialization.write(&output);
    
    Serial.print(leftInput);
    Serial.print(" ");
    Serial.print(rightInput);
    Serial.print(" ");
    Serial.print(leftTotal);
    Serial.print(" ");
    Serial.print(rightTotal);
    Serial.print(" ");
    Serial.print(leftOutput);
    Serial.print(" ");
    Serial.print(rightOutput);
    Serial.print(" ");
    Serial.print(requestLeft);
    Serial.print(" ");
    Serial.print(requestRight);
    Serial.print(" ");
    Serial.print("\n");
    
  }
}