from messages.arduino.Imu import Imu
from serialization.SerializationCoreBridge import SerializationCoreBridge
from messages.arduino.KvMessage import KvMessage, KvIntMessage, KvByteMessage, KvFloatMessage
from core.core import Core, TestSubscriber, TestPublisher
import time
from nav.ImuTransformationNode import ImuTransform
from socketio.socketio import FlaskBridge, FlaskBridgeParamNode
from messages.TestMessage import TestMessage
from nav.EnginesController import EnginesController, SpeedMode, DirectionMode, DirectionController, SpeedController
from engines.DifferentialEnginesControl import DifferentialEnginesControl
from messages.MagnetoMessage import MagnetoMessage
from messages.arduino.GenaralMessages import StringMessage
from messages.socket.GamepadMessage import GamepadMessage
import nav.calc.Calc
from test.FollowCards import FollowCards
from messages.MessagesUtils import ParamSyncNode, ParamSyncRule, MessageModeType
from messages.core.Message import ErrorMessage, EmptyMessage, ValueMessage, KeyValueMessage
from nodes.GamepadButtonsNode import GamepadButtonsNode

if __name__ == '__main__':
    core = Core()
    print(core.config)
    core.setDebug(True)
    core.registerTopic("/gamepad", GamepadMessage)
    core.registerTopic("/control/engines/enable", ValueMessage)
    core.registerTopic("/kvFromArduino", KvFloatMessage)
    core.registerTopic("/str", StringMessage)
    core.registerTopic(KvFloatMessage().topic, KvFloatMessage)
    syn = ParamSyncNode(core)
    core.registerTopic(Imu().topic, Imu)
    core.registerTopic("/test", TestMessage)
    core.registerTopic(KvByteMessage().topic, KvByteMessage)

    s = SerializationCoreBridge(core, core.getParam("/arduino/imu/com"), 115200)
    s.subscribeArduino(Imu().topic)
    s.subscribeCore(KvFloatMessage().topic, 4294901769, 4294901770)
    s.subscribeArduino("/kvFromArduino", 4294901769, 4294901770)
    s.subscribeArduino("/str", 4294901767, 4294901768)

    #s.subscribeCore(KvByteMessage().topic, 4294901763, 4294901764)
    #s.subscribeCore("/imu/magnetometer/calibrate", 4294901763, 4294901764)
    #s.subscribeCore(KvIntMessage().topic, 4294901765, 4294901766)
    s.start()

    # s2 = SerializationCoreBridge(core, core.getParam("/arduino/engines/com"), 115200)
    # s2.subscribeCore("/control/engines/output", 123456789, 987654321)
    # s2.start()
    #
    # s3 = SerializationCoreBridge(core, core.getParam("/arduino/rfid/com"), 115200)
    # s3.subscribeArduino("/str", 4294901767, 4294901768)
    # s3.start()

    ts = TestSubscriber(core, ErrorMessage().topic, True)
    ts1 = TestSubscriber(core, KvFloatMessage().topic, True)
    ts2 = TestSubscriber(core, "/kvFromArduino", True)
    #ts3 = TestSubscriber(core, "/str", True)
    ts4 = TestSubscriber(core, "/control/engines/enable", True)

    setList = [
        ParamSyncRule("/imu/magneto/xBias", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=1, mode=MessageModeType.SET.value)),
        ParamSyncRule("/imu/magneto/yBias", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=2, mode=MessageModeType.SET.value)),
        ParamSyncRule("/imu/magneto/zBias", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=3, mode=MessageModeType.SET.value)),
        ParamSyncRule("/imu/magneto/xScale", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=4, mode=MessageModeType.SET.value)),
        ParamSyncRule("/imu/magneto/yScale", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=5, mode=MessageModeType.SET.value)),
        ParamSyncRule("/imu/magneto/zScale", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=6, mode=MessageModeType.SET.value))
    ]

    getList = [
        ParamSyncRule("/imu/magneto/xBias", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=1, mode=MessageModeType.GET.value)),
        ParamSyncRule("/imu/magneto/yBias", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=2, mode=MessageModeType.GET.value)),
        ParamSyncRule("/imu/magneto/zBias", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=3, mode=MessageModeType.GET.value)),
        ParamSyncRule("/imu/magneto/xScale", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=4, mode=MessageModeType.GET.value)),
        ParamSyncRule("/imu/magneto/yScale", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=5, mode=MessageModeType.GET.value)),
        ParamSyncRule("/imu/magneto/zScale", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=6, mode=MessageModeType.GET.value))
    ]
    syn.addMessageParam("/setOffset", setList)
    syn.addMessageParam("/getOffset", getList)

    tr = ImuTransform(core)

    fb = FlaskBridge(core)
    fb.subscribeTopic("/imu")
    fb.subscribeTopic("/heading")
    fb.subscribeTopic("/control/engines")
    fb.subscribeTopic("/control/engines/output")
    fb.subscribeTopic("/followCards/speed")
    fb.subscribeTopic("/followCards/bearing")
    fb.subscribeTopic("/followCards/okAngle")
    fb.subscribeTopic("/timeRegulatorCoef")

    topics = [
        "/DirectionController/turnCoef",
        "/DirectionController/turnMode",
        "/DirectionController/TimeRegulator/decreaseMode",
        "/DirectionController/pid/p",
        "/DirectionController/pid/i",
        "/DirectionController/pid/d"
    ]

    fbpn = FlaskBridgeParamNode(core, fb, topics)

    time.sleep(10)
    print("Setting Magneto")
    core.publish(EmptyMessage("/setOffset"))
    time.sleep(30)
    print("Init complete")
    gp = GamepadButtonsNode(core)
    #msg = KvIntMessage()
    #msg.value = 100
    #core.publish(msg)
    dif = DifferentialEnginesControl(core)
    ec = EnginesController(core, dif)
    fc = FollowCards(core, ec)

    #set to manual control
    # m = KeyValueMessage(DirectionController.DIRECTION_CONTROLLER_MODE_TOPIC, DirectionMode.GAMEPAD_INPUT.value, 0)
    # core.publish(m)
    # m = KeyValueMessage(SpeedController.SPEED_CONTROLLER_MODE_TOPIC, SpeedMode.GAMEPAD_INPUT.value, 0)
    # core.publish(m)