from HeadInputMessage import HeadInputMessage
from HeadOutputMessage import HeadOutputMessage
from TestHidToFlask import BufferedInput
from messages.arduino.Imu import Imu
from nav.NavObjects import GeoPoint
from nodes.CameraNode import CameraNode
from nodes.General import PersistConfigNode, UnderControlNode, SendMessageDelayNode, TaskProcessor
from nodes.GpsParserNode import GpsParserNode, FakeGpsNode
from nodes.GpsPathOperationNode import GpsPathOperationNode
from serialization.SerializationCoreBridge import SerializationCoreBridge
from messages.arduino.KvMessage import KvMessage, KvIntMessage, KvByteMessage, KvFloatMessage
from core.core import Core, TestSubscriber, TestPublisher
import time
from nav.ImuTransformationNode import ImuTransform
from socketio.socketio import FlaskBridge, FlaskBridgeParamNode
from messages.TestMessage import TestMessage
from nav.EnginesController import EnginesController, SpeedMode, DirectionMode, DirectionController, SpeedController, \
    AdvancedEnginesController, SimpleEnginesController
from engines.DifferentialEnginesControl import DifferentialEnginesControl
from messages.MagnetoMessage import MagnetoMessage
from messages.arduino.GenaralMessages import StringMessage
from messages.socket.GamepadMessage import GamepadMessage
import nav.calc.Calc
from test.FollowCards import FollowCards
from messages.MessagesUtils import ParamSyncNode, ParamSyncRule, MessageModeType
from messages.core.Message import ErrorMessage, EmptyMessage, ValueMessage, KeyValueMessage
from nodes.GamepadButtonsNode import GamepadButtonsNode
from messages.arduino.BridgeMessage import ImprovedBridgeMessage
from RfidTest import StringMessageContains
from nav.AP import ObstacleAvoidanceNode, ObstacleAvoidanceNode2, ToPointNavigationNode, Navigate, NavTaskProcessor, \
    ObstacleAvoidanceNode3
from messages.GpsMessage import GpsMessage


class MCN:
    mapper = {
        3: "/control/engines/totalTicks",
        0: "/control/engines/totalTicksComplete",
        1: "/control/engines/frontTicks",
        2: "/control/engines/rearTicks"
    }

    def __init__(self, core: Core, fb: FlaskBridge):
        self.core = core
        self.fb = fb
        self.core.subscribe("/control/engines/fromArduino", self.process)
        self.fb.subscribeTopic(self.mapper[0])
        self.fb.subscribeTopic(self.mapper[1])
        self.fb.subscribeTopic(self.mapper[2])
        self.fb.subscribeTopic(self.mapper[3])

    def process(self, message: ImprovedBridgeMessage):
        message.topic = self.mapper[message.mode]
        self.core.publish(message)


class HeadingMessage(object):
    pass


if __name__ == '__main__':
    core = Core()
    print(core.config)
    core.setDebug(False)
    core.registerTopic("/gamepad", GamepadMessage)
    core.registerTopic("/control/engines/enable", ValueMessage)
    core.registerTopic("/kvFromArduino", KvFloatMessage)
    core.registerTopic("/str", StringMessage)
    core.registerTopic(KvFloatMessage().topic, KvFloatMessage)
    syn = ParamSyncNode(core)
    core.registerTopic(Imu().topic, Imu)
    core.registerTopic("/test", TestMessage)
    core.registerTopic(KvByteMessage().topic, KvByteMessage)

    s = SerializationCoreBridge(core, core.getParam("/arduino/imu/com"), 115200)
    s.subscribeArduino(Imu().topic)
    s.subscribeCore(KvFloatMessage().topic, 4294901769, 4294901770)
    s.subscribeArduino("/kvFromArduino", 4294901769, 4294901770)
    s.subscribeArduino("/str", 4294901767, 4294901768)

    s.subscribeCore(KvByteMessage().topic, 4294901763, 4294901764)
    s.subscribeCore("/imu/magnetometer/calibrate", 4294901763, 4294901764)
    s.subscribeCore(KvIntMessage().topic, 4294901765, 4294901766)
    s.start()

    s2 = SerializationCoreBridge(core, core.getParam("/arduino/engines/com"), 115200)
    # s2.subscribeCore("/control/engines/output", 123456789, 987654321)
    s2.subscribeCore("/control/engines/output2", 4294901761, 4294901762)
    s2.subscribeCore("/control/engines/pid", 4294901765, 4294901766)
    core.registerTopic("/control/engines/pid", KvFloatMessage)
    core.registerTopic("/control/engines/fromArduino", ImprovedBridgeMessage)
    s2.subscribeArduino("/control/engines/fromArduino", 4294901763, 4294901764)
    # tse = TestSubscriber(core, "/control/engines/output2", True)
    # tse = TestSubscriber(core, "/control/engines/fromArduino", True)
    s2.start()

    s3 = SerializationCoreBridge(core, core.getParam("/arduino/head/com"), 115200)
    core.registerTopic(HeadInputMessage().topic, HeadInputMessage)
    s3.subscribeArduino(HeadInputMessage().topic)
    s3.start()

    h = HeadOutputMessage()
    h.mode = 1
    h.preamble = 4294900001
    h.postamble = 4294900002
    h.angle = 90
    h.delay = 100
    s3.write(h)

    # s3 = SerializationCoreBridge(core, core.getParam("/arduino/rfid/com"), 115200)
    # core.registerTopic("/str", StringMessage)
    # s3.subscribeArduino("/str", 4294901767, 4294901768)
    # s3.start()
    # node = StringMessageContains(core, "/str")

    # s3 = SerializationCoreBridge(core, core.getParam("/arduino/rfid/com"), 115200)
    # s3.subscribeArduino("/str", 4294901767, 4294901768)
    # s3.start()

    ts = TestSubscriber(core, ErrorMessage().topic, True)
    ts1 = TestSubscriber(core, KvFloatMessage().topic, True)
    ts2 = TestSubscriber(core, "/kvFromArduino", True)
    # ts3 = TestSubscriber(core, "/str", True)
    ts4 = TestSubscriber(core, "/control/engines/enable", True)

    setList = [
        ParamSyncRule("/imu/magneto/xBias", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=1, mode=MessageModeType.SET.value)),
        ParamSyncRule("/imu/magneto/yBias", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=2, mode=MessageModeType.SET.value)),
        ParamSyncRule("/imu/magneto/zBias", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=3, mode=MessageModeType.SET.value)),
        ParamSyncRule("/imu/magneto/xScale", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=4, mode=MessageModeType.SET.value)),
        ParamSyncRule("/imu/magneto/yScale", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=5, mode=MessageModeType.SET.value)),
        ParamSyncRule("/imu/magneto/zScale", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=6, mode=MessageModeType.SET.value))
    ]

    getList = [
        ParamSyncRule("/imu/magneto/xBias", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=1, mode=MessageModeType.GET.value)),
        ParamSyncRule("/imu/magneto/yBias", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=2, mode=MessageModeType.GET.value)),
        ParamSyncRule("/imu/magneto/zBias", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=3, mode=MessageModeType.GET.value)),
        ParamSyncRule("/imu/magneto/xScale", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=4, mode=MessageModeType.GET.value)),
        ParamSyncRule("/imu/magneto/yScale", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=5, mode=MessageModeType.GET.value)),
        ParamSyncRule("/imu/magneto/zScale", "/kvFromArduino",
                      KvFloatMessage(preamble=123456789, postamble=987654321, key=6, mode=MessageModeType.GET.value))
    ]
    syn.addMessageParam("/setOffset", setList)
    syn.addMessageParam("/getOffset", getList)

    tr = ImuTransform(core)

    fb = FlaskBridge(core)
    fb.subscribeTopic("/imu")
    fb.subscribeTopic("/heading")
    fb.subscribeTopic("/bearing")
    fb.subscribeTopic("/control/engines")
    fb.subscribeTopic("/control/engines/output")
    fb.subscribeTopic("/control/engines/output2")
    fb.subscribeTopic("/followCards/speed")
    fb.subscribeTopic("/followCards/bearing")
    fb.subscribeTopic("/followCards/okAngle")
    fb.subscribeTopic("/timeRegulatorCoef")
    fb.subscribeTopic("/control/engines/fromArduino")
    fb.subscribeTopic("/obstacleHeading")
    # fb.subscribeTopic("/him")
    fb.subscribeTopic("/himb")
    fb.subscribeTopic("/obstacleAvoidance/status")
    fb.subscribeTopic("/obstacleBearing")

    def resender(message: GeoPoint):
        core.publish(GeoPoint("/position/destination", message.lat, message.lon))

    def roverPosition(message: GpsMessage):
        core.publish(GeoPoint(topic="/position/rover", lat=message.lat, lon=message.lon))

    core.registerTopic("/position/rover", GpsMessage)
    fb.subscribeTopic("/position/rover")
    fb.subscribeTopic("/position/destination")
    core.registerTopic("/destinationPoint", GeoPoint)
    core.subscribe("/destinationPoint", resender)
    core.subscribe("/gps", roverPosition)

    topics = [
        "/DirectionController/turnCoef",
        "/DirectionController/turnMode",
        "/DirectionController/TimeRegulator/decreaseMode",
        "/DirectionController/pid/p",
        "/DirectionController/pid/i",
        "/DirectionController/pid/d",
        "/followCards/angle1",
        "/followCards/angle2",
        "/followCards/bearing",
        "/controlEnginesPid/p",
        "/controlEnginesPid/i",
        "/controlEnginesPid/d",
        "/nav/okdistance",
        "/nav/minimalDistances",
        "/followCards/speedOverride",
    ]
    time.sleep(10)
    print("Setting Magneto")
    core.publish(EmptyMessage("/setOffset"))
    time.sleep(30)
    fbpn = FlaskBridgeParamNode(core, fb, topics)
    # aec = AdvancedEnginesController(core)
    sim = SimpleEnginesController(core)
    tp = TaskProcessor(core)
    sm = SendMessageDelayNode(core)
    mcn = MCN(core, fb)
    buf = BufferedInput(core, 3)
    obs = ObstacleAvoidanceNode3(core)
    tpn = ToPointNavigationNode(core)
    ntp = NavTaskProcessor(core)
    gpsn = GpsParserNode(core)
    # gf = FakeGpsNode(core)
    g = GpsPathOperationNode(core, fb)
    pcn = PersistConfigNode(core)
    nav = Navigate(core)
    ucn = UnderControlNode(core, fb)
    camn = CameraNode(core, s)

    def calibMegneto(message: EmptyMessage):
        msg = KvFloatMessage(preamble=123456789, postamble=987654321, key=7, mode=MessageModeType.GET.value)
        core.publish(msg)
    core.registerTopic("/calibMagneto", EmptyMessage)
    core.subscribe("/calibMagneto", calibMegneto)

    def onCalibOkProcessOffsets(message: EmptyMessage):
        print("### Setting up new magneto offsets ###")
        core.publish(EmptyMessage("/getOffset"))


    core.registerTopic("/saveMagnetoCoef", EmptyMessage)
    core.subscribe("/saveMagnetoCoef", onCalibOkProcessOffsets)

    # gp = GamepadButtonsNode(core)
    # msg = KvIntMessage()
    # msg.value = 100
    # core.publish(msg)
    # dif = DifferentialEnginesControl(core)
    # ec = EnginesController(core, dif)
    # fc = FollowCards(core, ec)

    # set to manual control
    # m = KeyValueMessage(DirectionController.DIRECTION_CONTROLLER_MODE_TOPIC, DirectionMode.GAMEPAD_INPUT.value, 0)
    # core.publish(m)
    # m = KeyValueMessage(SpeedController.SPEED_CONTROLLER_MODE_TOPIC, SpeedMode.GAMEPAD_INPUT.value, 0)
    # core.publish(m)

    # m = KvFloatMessage("/control/engines/pid",key=3,value=1);
    # core.publish(m)