import messages.core.Message
from messages.core.Message import EmptyMessage, ErrorMessage
from messages.arduino.GenaralMessages import StringMessage
from messages.arduino.KvMessage import KvMessage, KvFloatMessage, MessageModeType
from messages.MessagesUtils import MessageExpanderNode, ParamSyncNode, ParamSyncRule
from core.core import Core, TestSubscriber
from serialization.SerializationCoreBridge import SerializationCoreBridge
import time

if __name__ == '__main__':
    core = Core()
    #core.deb = False
    core.setDebug(False)
    core.registerTopic("/kvFromArduino", KvFloatMessage)
    core.registerTopic("/str", StringMessage)
    core.registerTopic(KvFloatMessage().topic, KvFloatMessage)
    exp = MessageExpanderNode(core)
    syn = ParamSyncNode(core)
    sb = SerializationCoreBridge(core, core.getParam("/arduino/imu/com"), 115200)
    sb.start()
    sb.subscribeCore(KvFloatMessage().topic, 4294901769, 4294901770)
    #sb.subscribeArduino("/kvFromArduino", 123456789, 987654321)
    sb.subscribeArduino("/kvFromArduino", 4294901769, 4294901770)
    sb.subscribeArduino("/str", 4294901767, 4294901768)
    ts = TestSubscriber(core, ErrorMessage().topic, True)
    ts1 = TestSubscriber(core, KvFloatMessage().topic, True)
    ts2 = TestSubscriber(core, "/kvFromArduino", True)
    ts3 = TestSubscriber(core, "/str", True)


    setList = [
        ParamSyncRule("/imu/magneto/xBias", "/kvFromArduino", KvFloatMessage(preamble=123456789,postamble=987654321, key=1, mode=MessageModeType.SET.value)),
        ParamSyncRule("/imu/magneto/yBias", "/kvFromArduino",KvFloatMessage(preamble=123456789, postamble=987654321, key=2, mode=MessageModeType.SET.value)),
        ParamSyncRule("/imu/magneto/zBias", "/kvFromArduino",KvFloatMessage(preamble=123456789, postamble=987654321, key=3, mode=MessageModeType.SET.value)),
        ParamSyncRule("/imu/magneto/xScale", "/kvFromArduino",KvFloatMessage(preamble=123456789, postamble=987654321, key=4, mode=MessageModeType.SET.value)),
        ParamSyncRule("/imu/magneto/yScale", "/kvFromArduino",KvFloatMessage(preamble=123456789, postamble=987654321, key=5, mode=MessageModeType.SET.value)),
        ParamSyncRule("/imu/magneto/zScale", "/kvFromArduino",KvFloatMessage(preamble=123456789, postamble=987654321, key=6, mode=MessageModeType.SET.value))
    ]

    getList = [
        ParamSyncRule("/imu/magneto/xBias", "/kvFromArduino",KvFloatMessage(preamble=123456789, postamble=987654321, key=1, mode=MessageModeType.GET.value)),
        ParamSyncRule("/imu/magneto/yBias", "/kvFromArduino",KvFloatMessage(preamble=123456789, postamble=987654321, key=2, mode=MessageModeType.GET.value)),
        ParamSyncRule("/imu/magneto/zBias", "/kvFromArduino",KvFloatMessage(preamble=123456789, postamble=987654321, key=3, mode=MessageModeType.GET.value)),
        ParamSyncRule("/imu/magneto/xScale", "/kvFromArduino",KvFloatMessage(preamble=123456789, postamble=987654321, key=4, mode=MessageModeType.GET.value)),
        ParamSyncRule("/imu/magneto/yScale", "/kvFromArduino",KvFloatMessage(preamble=123456789, postamble=987654321, key=5, mode=MessageModeType.GET.value)),
        ParamSyncRule("/imu/magneto/zScale", "/kvFromArduino",KvFloatMessage(preamble=123456789, postamble=987654321, key=6, mode=MessageModeType.GET.value))
    ]

    syn.addMessageParam("/setOffset", setList)
    syn.addMessageParam("/getOffset", getList)

    time.sleep(10)
    core.publish(KvFloatMessage(preamble=123456789, postamble=987654321, key=7, mode=MessageModeType.GET.value))
    time.sleep(60)
    core.publish(EmptyMessage("/getOffset"))
    #core.publish(EmptyMessage("/setOffset"))
    while True:
        #print(syn.isProcessed())
        if syn.isProcessed():
            break
        time.sleep(0.1)

    time.sleep(20)
    #core.setParam("/arduino/imu/com", "/dev/ttyUSB2")
    #core.setParam("/arduino/imu/baud", 115200)
    #core.setParam("/arduino/engines/com", "/dev/ttyUSB0")
    #core.setParam("/arduino/engines/baud", 115200)
    print("##############################")
    print(core.config)
    #print("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@")
    #print(core.getParam("/imu/magneto/xOffset"))
    #print(core.getParam("/imu/magneto/yOffset"))
    print("##############################")
    core.stop()