from core.core import Core, TestSubscriber
from nav.calc.Buffers import CircularBuffer
from serialization.SerializationCoreBridge import SerializationCoreBridge
from serialization.SerializationSerialConnection import SerializationSerialConnection
from HeadInputMessage import HeadInputMessage
from HeadInputMessageListener import HeadMessageListener
from UltrasonicParamsMessage import UltrasonicParamsMessage
from HeadOutputMessage import HeadOutputMessage
import time
from math import *
# import numpy as np
import statistics

from socketio.socketio import FlaskBridge


class Kalman:
    measurement_sig = 4.
    motion_sig = 2.
    mu = 0.
    sig = 10000.

    def __init__(self):
        self.mu = Kalman.mu
        self.sig = Kalman.sig
        self.measurement_sig = Kalman.measurement_sig
        self.mu_out = 0
        self.sig_out = 0

    # the update function
    def update(mean1, var1, mean2, var2):
        ''' This function takes in two means and two squared variance terms,
            and returns updated gaussian parameters.'''
        # Calculate the new parameters
        new_mean = (var2 * mean1 + var1 * mean2) / (var2 + var1)
        new_var = 1 / (1 / var2 + 1 / var1)

        return [new_mean, new_var]

    def updateInstance(self, value):
        self.mu, self.sig = Kalman.update(self.mu, self.sig, value, self.measurement_sig)

    # the motion update/predict function
    def predict(mean1, var1, mean2, var2):
        ''' This function takes in two means and two squared variance terms,
            and returns updated gaussian parameters, after motion.'''
        # Calculate the new parameters
        new_mean = mean1 + mean2
        new_var = var1 + var2

        return [new_mean, new_var]

    def f(mu, sigma2, x):
        ''' f takes in a mean and squared variance, and an input x
           and returns the gaussian value.'''
        coefficient = 1.0 / sqrt(2.0 * pi * sigma2)
        exponential = exp(-0.5 * (x - mu) ** 2 / sigma2)
        return coefficient * exponential


class KalmanInput:

    def __init__(self, core: Core):
        self.buffer = {}
        self.core = core
        self.message = HeadInputMessage("/himb")
        self.core.subscribe("/him", self.process)

    def process(self, message : HeadInputMessage):
        if message.ultrasonic > 200:
            message.ultrasonic = 200
        if message.servo not in self.buffer:
            kalman = Kalman()
        else:
            kalman = self.buffer[message.servo]
        kalman.updateInstance(message.ultrasonic)
        self.buffer[message.servo] = kalman
        self.message.servo = message.servo
        try:
            self.message.lidarSignal = message.ultrasonic
            self.message.ultrasonic = kalman.mu
            self.message.lidar = kalman.sig
        except:
            pass
        self.core.publish(self.message)


class BufferedInput:

    def __init__(self, core: Core, bufferSize: int):
        self.buffer = {}
        self.core = core
        self.bufferSize = bufferSize
        self.message = HeadInputMessage("/himb")
        self.core.subscribe("/him", self.process)
        self.counter = 0
        self.offset = 10

    def process(self, message : HeadInputMessage):
        # if message.servo in self.buffer:
        #     print(len(self.buffer[message.servo].buffer))
        if message.ultrasonic > 100:
            message.ultrasonic = 100
        if message.servo not in self.buffer:
            buffer = CircularBuffer(self.bufferSize)
        else:
            buffer = self.buffer[message.servo]
        buffer.append(message.ultrasonic)
        self.buffer[message.servo] = buffer
        self.message.servo = message.servo
        try:
            self.message.lidarSignal = buffer.mean()
            self.message.ultrasonic = buffer.mod()
            self.message.lidar = buffer.med()
        except:
            pass
        self.message.irCode = ""
        for key in self.buffer:
            buff = self.buffer[key]
            try:
                mean = statistics.mean([buff.mean(), buff.mod(), buff.med()])
            except:
                mean = -1
            # mean = mean / 10
            self.message.irCode = "{} {}".format(self.message.irCode, int(mean))

        # print(self.message.irCode)
        self.core.publish(self.message)



if __name__ == '__main__':


    core = Core()

    core.registerTopic(HeadInputMessage().topic, HeadInputMessage)

    s = SerializationCoreBridge(core, "COM10", 115200)
    s.subscribeArduino(HeadInputMessage().topic)
    s.start()

    fb = FlaskBridge(core)
    fb.subscribeTopic("/himb")

    buf = BufferedInput(core, 3)
    # kal = KalmanInput(core)

    # TestSubscriber(core, HeadInputMessage().topic, True)

    h = HeadOutputMessage()
    h.mode = 1
    h.preamble = 4294900001
    h.postamble = 4294900002
    h.angle = 90
    h.delay = 100
    s.write(h)