import time
import threading
from utils.TimeUtils import getCurrentTimeMillis
import dpath.util
from config.config import saveJson, loadJson
from enum import Enum

# kazdy subscriber bude definovat pozadovana temata a zda jde o black list nebo whitelist
# core by mel zustat co nejcistsi, takze se kazde tema bude obsahovat subscribery
# core bude obsahovat pouze topic a jejich subscribery
# pri subscriberu se naleyitosti prepocitaji a core updatuje svuj seznam topicu a subscriberu
# trate je synchronized a jeho update je lockovan

class Core:
    allTopics = "/"
    configPath = "config.json"
    configParamPrefix = "parameter"
    ignore = [
        "/destinationPoint",
        "/nav/onPoint",
        "/nav/complete",
        "/nav/okdistance",
        "/nav/tasks",
        "/pathOperation/path",
        "/restDistance",
        "/taskProcessor/complete",
        "/taskProcessor/tasks",
        "/sendMessageDelay/activate",
        "/sendMessageDelay/elapsed",

    ]

    def __init__(self):
        self.config = loadJson(self.configPath)
        self.alive = True
        self.deb = False
        self.lock = threading.Lock()
        self.subscribers = {}
        self.params = {}
        self.services = {}
        self.rate = {}
        self.trate = {}
        self.lastSec = 0
        self.debugInterval = 1
        self.t = threading.Thread(target=self.debug)
        self.t.start()
        self.mapper = {}
        self.onParamChangeCallback = {}

    def stop(self):
        saveJson(self.configPath, self.config)
        self.alive = False

    def persistConfig(self):
        saveJson(self.configPath, self.config)

    def setDebug(self, deb):
        """
        Zapnuti vypnuti debugovacich zprav
        :param debug:
        :return:
        """
        self.lock.acquire()
        self.deb = deb
        self.lock.release()

    def debug(self):
        while self.alive:
            if self.deb:
                self.lock.acquire()
                for x in self.trate.keys():
                    print(x+": "+str(self.trate[x])+"\t", end='')
                self.lock.release()
                print()
                time.sleep(self.debugInterval)
            else:
                time.sleep(self.debugInterval)

    def logRate(self, topic):
        if topic in self.rate.keys():
            self.rate[topic] += 1
        else:
            self.rate[topic] = 0
        if self.lastSec + 1000 < getCurrentTimeMillis():
            self.lastSec = getCurrentTimeMillis()
            self.lock.acquire()
            for x in self.rate.keys():
                self.trate[x] = self.rate[x]
                self.rate[x] = 0
            # for x in set(self.trate.keys()).difference(set(self.rate.keys())):
            #     self.trate[x] = 0
            self.lock.release()

    def publish(self, message):
        if message.topic in self.ignore:
            print("CORE TOPIC {}".format(message.topic))
        self.logRate(message.topic)
        #print("Publishing "+message.topic+" "+str(message))
        if message.topic in self.subscribers:
            for x in self.subscribers[message.topic]:
                #x.publish(message)
                x(message)
        if self.allTopics in self.subscribers:
            for x in self.subscribers[self.allTopics]:
                x(message)

    def subscribe(self, topic, callback):
        if topic in self.subscribers:
            list = self.subscribers[topic]
            list.append(callback)
        else:
            list = []
            list.append(callback)
            self.subscribers[topic] = list

    def subscribeAll(self, callback):
        self.subscribe(self.allTopics, callback)

    def unsubscribe(self, callback):
        for topic in self.subscribers.values():
            if callback in topic:
                topic.remove(callback)

    def getParam(self, param):
        try:
            return dpath.util.get(self.config, self.configParamPrefix+param)
        except KeyError:
            return None #fuj

    def setParam(self, param, value):
        dpath.util.new(self.config, self.configParamPrefix+param, value)
        print("{} {} {}".format(param, value, self.onParamChangeCallback))
        if param in self.onParamChangeCallback:
            for callback in self.onParamChangeCallback[param]:
                callback(value)

    def addOnParamChangeCallback(self, param, callback):
        callbacks = self.onParamChangeCallback[param]
        if callbacks is None:
            callbacks = []
        callbacks.append(callback)
        self.onParamChangeCallback[param] = callbacks

    def callService(self, *params):
        pass

    def registerService(self, service):
        pass

    def multipleArg(self, *a):
        print(a[0])

    def multipleArg2(self, **a):
        print(a["x"])

    def method(self, method, *a):
        print(method(a[0], a[1]))

    def registerService(self, name, method):
        pass

    def callService(self, name, *args):
        pass

    def registerTopic(self, topic, clazz, **kwargs):
        self.mapper[topic] = (clazz, kwargs)

    def getTopicClass(self, topic):
        if topic in self.mapper:
            return self.mapper[topic][0]
        else:
            return None

    def getTopicKwargs(self, topic):
        if topic in self.mapper:
            return self.mapper[topic][1]
        else:
            return None

    def getTopic(self, base: str, *sub) -> str:
        return "/".join((base, "/".join(sub)))

class CoreSender:

    def __init__(self, core: Core):
        self.core = core

    def dataUpdate(self, data):
        self.core.publish(data)

class TestPublisher:
    def __init__(self, core: Core, topic, sleep):
        """
        Testovaci publisher
        :param core:
        :param topic:
        :param sleep:
        """
        self.core = core
        self.topic = topic
        self.sleep = sleep
        self.t = threading.Thread(target=self.run)
        self.t.start()

    def run(self):
        while True:
            msg = E()
            msg.topic = self.topic
            msg.message = "nejaka adekvatne dlouha zprava protoze ty data taky budou obsahovat nejaka data a mozna i takhle dlouha nebo delsi rikEvil"
            self.core.publish(msg)
            time.sleep(self.sleep)

class TestSubscriber:
    def __init__(self, core: Core, topic, debug):
        self.core = core
        self.topic = topic
        self.debug = debug
        self.core.subscribe(self.topic, self.customCallback)

    def publish(self, message):
        if self.debug:
            pass
            #print("Topic: "+message.topic+"\t"+str(message))
            #print(message)

    def customCallback(self, message):
        if self.debug:
            print("Topic: "+message.topic+"\t"+str(message))


class X:

    def publish(self, message):
        pass
        #print("In msg: "+str(message))
        #print(type(message))

    def add(self, a, b):
        return a+b

class E:
    pass


class Node:

    def __init__(self, core: Core):
        self.core = core

class Position(Enum):
    Front = 0
    Center = 1
    Back = 2
    Left = 3
    Right = 4

if __name__ == '__main__':
    c = Core()
    c.setDebug(False)
    #c.multipleArg(1,2,3)
    #c.multipleArg2(x=1,y=2)
    #x = X()
    #c.method(x.add,5 ,3)

    x = X()
    y = X()
    c.subscribe("/", x)
    c.subscribe("topic1", y)
    msg = E()
    msg.topic = "topi"
    msg.message = "Ahoj"
    c.publish(msg)
    print("")
    m = E()
    m.topic = "/head"
    m.message = "Achooooj"
    m1 = E()
    m1.topic = "/headx"
    m1.message = "Achooooj"
    #c.unsubscribe(x)
    #c.unsubscribe(x)
    c.publish(msg)

    tp = TestPublisher(c, "testTopic0", 0.1)
    tr = TestSubscriber(c, "testTopic0", True)
    #c.unsubscribe(tr.customCallback)

    for x in range(100000000):
        c.publish(msg)
        c.publish(m)
        c.publish(m1)
        #c.debug()
        #time.sleep(0.0001)
