from serialization.SerializationData import SerializationData
from serialization.SerializationInputStream import SerializationInputStream
from serialization.SerializationOutputStream import SerializationOutputStream

class BridgeMessage(SerializationData):

    def __init__(self, topic="/control/engines/output", mode=0, left=0, right=0, enable=0, *args, **kwargs):
        super().__init__()
        self.topic = topic
        self.mode = mode
        self.left = left
        self.right = right
        self.enable = enable

    def __str__(self):
        #return "Mode: "+str(self.mode)+"\tLeft: "+str(self.left)+"\tRight: "+str(self.right)
        return "Mode: {}\tLeft: {}\tRight: {}\tEnable: {}".format(self.mode, self.left, self.right, self.enable)

    def getDataSize(self):
        return 4

    def readData(self, input: SerializationInputStream):
        self.mode = input.readByte()
        self.left = input.readByte()
        self.right = input.readByte()
        self.enable = input.readByte()

    def writeData(self, output: SerializationOutputStream):
        output.writeByte(self.mode)
        output.writeByte(self.left)
        output.writeByte(self.right)
        output.writeByte(self.enable)


class ImprovedBridgeMessage(SerializationData):

    def __init__(self, topic="/control/engines/output2", left=0, right=0,mode=0, enable=0,*args, **kwargs):
        super().__init__(topic=topic)
        self.left = left
        self.right = right
        self.mode = mode
        self.enable = enable

    def __str__(self):
        return "Mode: {}\tLeft: {}\tRight: {}\tEnable: {}".format(self.mode, self.left, self.right, self.enable)

    def getDataSize(self):
        return 2*4+2

    def readData(self, input: SerializationInputStream):
        self.left = input.readFloat()
        self.right = input.readFloat()
        self.mode = input.readByte()
        self.enable = input.readByte()
        
    def writeData(self, output: SerializationOutputStream):
        output.writeFloat(self.left)
        output.writeFloat(self.right)
        output.writeByte(self.mode)
        output.writeByte(self.enable)


