from serialization.SerializationData import SerializationData
from serialization.SerializationInputStream import SerializationInputStream
from serialization.SerializationOutputStream import SerializationOutputStream
from messages.core.Message import Message
from utils.Structures import Vector
import math


class Imu(SerializationData):

    def __init__(self, topic="/imuRaw", preamble=4294901761, postamble=4294901762, *args, **kwargs):
        super().__init__(topic, preamble, postamble, *args , **kwargs)
        self.acc = Vector()
        self.gyr = Vector()
        self.mag = Vector()

    def __str__(self):
        return "Acc: {}\tGyr: {}\tMag: {}".format(self.acc, self.gyr, self.mag)

    def getDataSize(self):
        return 3*3*4

    def getHeading(self):
        return self.mag.x
        #return 180 - math.atan2(self.mag.x, self.mag.y) / 0.0174532925

    def readData(self, input: SerializationInputStream):
        self.acc = Vector(input.readFloat(), input.readFloat(), input.readFloat())
        self.gyr = Vector(input.readFloat(), input.readFloat(), input.readFloat())
        self.mag = Vector(input.readFloat(), input.readFloat(), input.readFloat())

    def writeData(self, output: SerializationOutputStream):
        output.writeFloat(self.acc.x)
        output.writeFloat(self.acc.y)
        output.writeFloat(self.acc.z)
        output.writeFloat(self.gyr.x)
        output.writeFloat(self.gyr.y)
        output.writeFloat(self.gyr.z)
        output.writeFloat(self.mag.x)
        output.writeFloat(self.mag.y)
        output.writeFloat(self.mag.z)


class ImuMessage(Message):

    def __init__(self, topic="/imu", heading = 0, *args, **kwargs):
        super().__init__(topic, *args, **kwargs)
        self.heading = heading

    def __str__(self):
        return "Heading: {}".format(self.heading)