from serialization.SerializationData import SerializationData
from serialization.SerializationInputStream import SerializationInputStream
from serialization.SerializationOutputStream import SerializationOutputStream
import enum


class KvMessage(SerializationData):

    def __init__(self, topic="/", preamble=0, postamble=0,key = 0, mode = 0, value = 0, *args, **kwargs):
        super().__init__(topic, preamble, postamble, *args, **kwargs)
        self.key = key
        self.mode = mode
        self.value = value

    def __str__(self):
        return "Key: {}\tMode: {}\tValue: {}".format(self.key, self.mode, self.value)

    def getDataSize(self):
        return 5

    def readData(self, input: SerializationInputStream):
        self.key = input.readInteger()
        self.mode = input.readByte()

    def writeData(self, output: SerializationOutputStream):
        output.writeInteger(self.key)
        output.writeByte(self.mode)


class KvByteMessage(KvMessage):

    def __init__(self, topic="/kvbm", preamble=4294901763, postamble=4294901764,key = 0, mode = 0, value = 0, *args, **kwargs):
        super().__init__(topic, preamble, postamble,key,mode,value, *args, **kwargs)

    def getDataSize(self):
        return super().getDataSize()+1

    def readData(self, input: SerializationInputStream):
        super().readData(input)
        self.value = input.readByte()

    def writeData(self, output: SerializationOutputStream):
        super().writeData(output)
        output.writeByte(self.value)


class KvIntMessage(KvMessage):

    def __init__(self, topic="/kvim", preamble=4294901765, postamble=4294901766,key = 0, mode = 0, value = 0, *args, **kwargs):
        super().__init__(topic, preamble, postamble,key,mode,value, *args, **kwargs)

    def getDataSize(self):
        return super().getDataSize()+4

    def readData(self, input: SerializationInputStream):
        super().readData(input)
        self.value = input.readInteger()

    def writeData(self, output: SerializationOutputStream):
        super().writeData(output)
        output.writeInteger(self.value)


class KvFloatMessage(KvMessage):

    def __init__(self, topic="/kvfm", preamble=4294901767, postamble=4294901768,key = 0, mode = 0, value = 0, *args, **kwargs):
        super().__init__(topic, preamble, postamble,key,mode,value, *args, **kwargs)

    def getDataSize(self):
        return super().getDataSize()+4

    def readData(self, input: SerializationInputStream):
        super().readData(input)
        self.value = input.readFloat()

    def writeData(self, output: SerializationOutputStream):
        super().writeData(output)
        output.writeFloat(self.value)


class MessageModeType(enum.Enum):
    SET = 0
    GET = 1
    REQUEST = 2
    RESPONSE = 3