from messages.arduino.BridgeMessage import ImprovedBridgeMessage
from nodes.CameraNode import CameraPosition, CameraNode, CameraTask
from nodes.General import Task, SendMessageDelayNode, Tasks
from utils.GeneralUtils import ValueWithOffset
from utils.ProgressUtils import Progress
from nav.NavObjects import GeoPoint, WayPoint, Path
from nodes.RemoteValue import ParamValue
from core.core import Core, TestSubscriber, TestPublisher
from HeadInputMessage import HeadInputMessage
from messages.MagnetoMessage import MagnetoMessage
from nodes.RemoteValue import ParamValue
from nav.calc.Calc import inRange, addSameSign, AngleCalc, calculateAngle, calculateDistance
import threading
import time
from nav.calc.Calc import addSameSign
from serialization.SerializationCoreBridge import SerializationCoreBridge
from nav.NavObjects import Direction
from messages.core.Message import ValueMessage, EmptyMessage
from messages.GpsMessage import GpsMessage
from nav.NavObjects import GeoPoint
from utils.ConvertUtils import byteToBoolean
from messages.MessagesUtils import MessagesEqual


class Navigate:
    PUBLISH_DESTINATION_TOPIC = "/destinationPoint"
    PUBLISH_ON_POINT = "/nav/onPoint"
    PUBLISH_COMPLETE = "/nav/complete"
    PARAM_OK_DISTANCE = "/nav/okdistance"
    PARAM_TASKS_ENABLE = "/nav/tasks"

    REQUIRED_PATH_TOPIC = "/pathOperation/path"
    REQUIRED_REST_DISTANCE = "/restDistance"
    REQUIRED_TASKS_COMPLETE = "/taskProcessor/complete"

    def __init__(self, core: Core):
        self.core = core
        self.path: Path
        self.okDistance: ParamValue
        self.onPointWait: ParamValue
        self.progress: ParamValue
        self.home: WayPoint
        self.nav = ToPointNavigationNode
        self.path = Path()
        self.pointer = 0
        self.core.registerTopic(self.PUBLISH_ON_POINT, EmptyMessage)
        self.core.registerTopic(self.PUBLISH_COMPLETE, EmptyMessage)
        self.core.registerTopic(self.PUBLISH_DESTINATION_TOPIC, WayPoint)
        self.core.subscribe(self.REQUIRED_REST_DISTANCE, self.processRestDistance)
        self.core.subscribe(self.REQUIRED_PATH_TOPIC, self.startAP)
        if byteToBoolean(self.core.getParam(self.PARAM_TASKS_ENABLE)):
            print("NAV_NEXT_POINT_ON_TASKS_DONE")
            self.core.subscribe(self.REQUIRED_TASKS_COMPLETE, self.nextPoint)

    def processRestDistance(self, message: ValueMessage):
        # print("Rest distance {} {}".format(message.value, self.core.getParam(self.PARAM_OK_DISTANCE)))
        if message.value < self.core.getParam(self.PARAM_OK_DISTANCE):
            self.core.publish(EmptyMessage(self.PUBLISH_ON_POINT))
            print("NAV_ON_POINT")
            if not byteToBoolean(self.core.getParam(self.PARAM_TASKS_ENABLE)):
                self.nextPoint(EmptyMessage())

    def nextPoint(self, message: EmptyMessage):
        print("NAV_NEXT_POINT")
        if len(self.path.waypoints) > self.pointer:
            self.processWaypoint(self.path.waypoints[self.pointer])
            self.pointer += 1
        else:
            print("NAV_COMPLETE")
            self.core.publish(EmptyMessage(self.PUBLISH_COMPLETE))

    def processWaypoint(self, waypoint: WayPoint):
        waypoint.topic = self.PUBLISH_DESTINATION_TOPIC
        self.core.publish(waypoint)

    def startAP(self, message: Path):
        self.path = message
        self.pointer = 0
        self.nextPoint(EmptyMessage())




class ToPointNavigationNode:
    SUBSCRIBE_GPS_TOPIC = "/gps"
    SUBSCRIBE_DESTINATION_TOPIC = "/destinationPoint"
    PUBLISH_BEARING_TOPIC = "/bearing"
    PUBLISH_DISTANCE_TOPIC = "/restDistance"

    def __init__(self, core: Core):
        self.core = core
        self.destination = GeoPoint()
        self.validDestination = False
        self.core.registerTopic(self.PUBLISH_BEARING_TOPIC, MagnetoMessage)
        self.core.registerTopic(self.PUBLISH_DISTANCE_TOPIC, ValueMessage)
        self.outputBearing = MagnetoMessage(self.PUBLISH_BEARING_TOPIC)
        self.outputRestDistance = ValueMessage(self.PUBLISH_DISTANCE_TOPIC)
        self.core.subscribe(self.SUBSCRIBE_GPS_TOPIC, self.processGps)
        self.core.subscribe(self.SUBSCRIBE_DESTINATION_TOPIC, self.processNavPoint)

    def processNavPoint(self, message: WayPoint):
        self.destination = message
        self.validDestination = True

    def processGps(self, message: WayPoint):
        if not self.validDestination:
            return
        current = GeoPoint(lat=message.lat, lon=message.lon)
        self.outputRestDistance.value = calculateDistance(current, self.destination)
        self.outputBearing.heading = calculateAngle(current, self.destination)
        self.core.publish(self.outputRestDistance)
        self.core.publish(self.outputBearing)
        # print("AP {} {} {} {} {} {}".format(current.lat,current.lon,self.destination.lat, self.destination.lon, self.outputBearing.heading, self.outputRestDistance.value))


class NavTaskProcessor:
    PROVIDE_TASKS = "/taskProcessor/tasks"

    REQUIRE_DESTINATION = "/destinationPoint"
    REQUIRE_ON_POINT = "/nav/onPoint"

    mapper = {}

    def __init__(self, core: Core):
        self.core = core
        self.tasks = None
        self.core.subscribe(self.REQUIRE_DESTINATION, self.processDestinationPoint)

    def processDestinationPoint(self, message: WayPoint):
        print("NAV_TASK_PROCESSOR_WAYPOINT")
        self.core.subscribe(self.REQUIRE_ON_POINT, self.processOnPoint)
        if message.name in self.mapper:
            self.tasks = self.mapper[message.name]
        else:
            # REQUIRED_DEFAULT_MESSAGE = "/sendMessageDelay/activate"
            # PROVIDED_DEFAULT_MESSAGE = "/sendMessageDelay/elapsed"
            # self.tasks = [
            #     Task(ValueMessage(REQUIRED_DEFAULT_MESSAGE, 5), EmptyMessage(PROVIDED_DEFAULT_MESSAGE), MessagesEqual.getTopicMessageEqual)
            # ]
            exp = EmptyMessage(CameraNode.PROVIDED_CAMERA_TASK_COMPLETE)
            equ = MessagesEqual.getTopicMessageEqual
            self.tasks = [
                Task(CameraPosition(pan=90, titl=0), exp, equ),
                Task(CameraTask(width=800, height=600, count=1), exp, equ),
                Task(CameraPosition(pan=90, titl=90), exp, equ),
                Task(CameraTask(width=800, height=600, count=1), exp, equ),
                Task(CameraPosition(pan=90, titl=180), exp, equ),
                Task(CameraTask(width=800, height=600, count=1), exp, equ)
            ]

    def processOnPoint(self, message: EmptyMessage):
        print("NAV_TASK_PROCESSOR_START_TASKS")
        self.core.unsubscribe(self.processOnPoint)
        self.core.publish(Tasks(self.PROVIDE_TASKS, self.tasks))


class AngleDirection:

    def __init__(self, angle=0, direction : Direction = Direction.Center):
        self.angle = angle
        self.direction = direction

    def getAngleValue(self):
        if self.direction == Direction.Center:
            return 0
        if self.direction == Direction.Left:
            return self.angle*-1
        return self.angle

    def __str__(self):
        return "Direction {}\tAngle{}".format(self.direction, self.angle)

class DirectionProcessor:
    dirSwitchMap = {
        Direction.Center: Direction.Center,
        Direction.Left: Direction.Right,
        Direction.Right: Direction.Left
    }

    def __init__(self):
        self.mapper = {}
        self.reset()

    def reset(self):
        self.mapper = {
            Direction.Left : 0,
            Direction.Center : 0,
            Direction.Right : 0
        }

    def append(self, angleDirection: AngleDirection):
        self.mapper[angleDirection.direction] += angleDirection.angle

    def directionBalance(self) -> Direction:
        if self.mapper[Direction.Left] == self.mapper[Direction.Right]:
            return Direction.Center
        if self.mapper[Direction.Left] < self.mapper[Direction.Right]:
            return Direction.Right
        else:
            return Direction.Left

    def directionAngleBalance(self) -> AngleDirection:
        direction = self.directionBalance()
        if direction == Direction.Center:
            return AngleDirection()
        return AngleDirection(abs(self.mapper[Direction.Left] - self.mapper[Direction.Right]), direction)

    @staticmethod
    def switchDirection(direction : Direction) -> Direction:
        return DirectionProcessor.dirSwitchMap[direction]


class ObstacleAvoidanceNode3:
    PARAM_DETECTION_OFFSET = "/obstacleAvoidance/detectionOffset"
    PARAM_0_COEF = "/obstacleAvoidance/coef0"
    PARAM_45_COEF = "/obstacleAvoidance/coef45"
    PARAM_90_COEF = "/obstacleAvoidance/coef90"
    PROVIDE_BEARING_TOPIC = "/obstacleBearing"
    PROVIDE_STATUS_TOPIC = "/obstacleAvoidance/status"
    REQUIRED_TICKS = "/control/engines/totalTicks"
    PARAM_DIFFERENCE_MODE = "/obstacleAvoidance/diffMode"
    PARAM_DIFFERENCE_TICKS_OFFSET = "/obstacleAvoidance/diffTicksOffset"
    PARAM_DIFFERENCE_STEP_OFFSET = "/obstacleAvoidance/diffStepOffset"

    def __init__(self, core: Core):
        self.core = core
        self.buffer = {}
        self.mapper = {
            0: -90,
            50: -45,
            90: 0,
            140: 45,
            180: 90
        }
        self.coefMapper = {
            -90: self.core.getParam(self.PARAM_90_COEF),
            -45: self.core.getParam(self.PARAM_45_COEF),
            0: self.core.getParam(self.PARAM_0_COEF),
            45: self.core.getParam(self.PARAM_45_COEF),
            90: self.core.getParam(self.PARAM_90_COEF)
        }
        self.core.subscribe("/bearing", self.processBearing)
        self.core.subscribe("/himb", self.processDistance)
        self.core.registerTopic(self.PROVIDE_STATUS_TOPIC, ValueMessage)
        self.core.registerTopic(self.PROVIDE_BEARING_TOPIC, MagnetoMessage)
        self.lock = threading.Lock()
        self.status = ""
        self.lastCompensation = 0
        self.diffMode = self.core.getParam(self.PARAM_DIFFERENCE_MODE)
        if self.diffMode == 1:
            self.leftTicks = ValueWithOffset(setPoint=self.core.getParam(self.PARAM_DIFFERENCE_TICKS_OFFSET))
            self.rightTicks = ValueWithOffset(setPoint=self.core.getParam(self.PARAM_DIFFERENCE_TICKS_OFFSET))
            self.core.subscribe(self.REQUIRED_TICKS, self.processTicks)

    def processLastCompensation(self):
        if self.lastCompensation > 0:
            self.lastCompensation = self.lastCompensation - self.core.getParam(self.PARAM_DIFFERENCE_STEP_OFFSET)
            if self.lastCompensation < 0:
                self.lastCompensation = 0
        if self.lastCompensation < 0:
            self.lastCompensation = self.lastCompensation + self.core.getParam(self.PARAM_DIFFERENCE_STEP_OFFSET)
            if self.lastCompensation > 0:
                self.lastCompensation = 0

    def processTicks(self, message: ImprovedBridgeMessage):
        if self.leftTicks.update(message.left) or self.rightTicks.update(message.right):
            self.processLastCompensation()
            self.leftTicks.offset = message.left
            self.rightTicks.offset = message.right

    def publishStatus(self):
        self.core.publish(ValueMessage(self.PROVIDE_STATUS_TOPIC, self.status))
        print(self.status)

    def getDistance(self, angle):
        output = None
        self.lock.acquire()
        output = self.buffer.get(angle)
        self.lock.release()
        return output

    def calcCompensation(self, angle):
        offset = self.core.getParam(self.PARAM_DETECTION_OFFSET)
        difference = offset - self.getDistance(angle)
        if difference > 0:
            return difference * self.coefMapper[angle]
        return 0

    def processFrontCompensationSign(self, compensation):
        left = self.getDistance(-90) + self.getDistance(-45)
        right = self.getDistance(90) + self.getDistance(45)
        if right <= left:
            return compensation*-1
        return compensation

    def calcCompensationAll(self):
        comp = self.calcCompensation(0)
        if comp > 0:
            comp = self.processFrontCompensationSign(comp)
            self.status += "Front {} ".format(int(comp))
            return comp
        else:
            self.status += "Front {} ".format(int(0))

        left = self.calcCompensation(-45) + self.calcCompensation(-90)
        right = (self.calcCompensation(45) + self.calcCompensation(90))*-1
        self.status += "L {} R {} ".format(int(left), int(right))
        comp = left + right
        return comp


    def processBearing(self, message: MagnetoMessage):
        self.status = ""
        if self.diffMode == 2:
            self.processLastCompensation()

        compensation = self.calcCompensationAll()
        if compensation == 0:
            compensation = self.lastCompensation
        # if abs(compensation) < abs(self.lastCompensation):
        #     compensation = self.lastCompensation

        output = message.heading + compensation
        self.lastCompensation = compensation
        self.status += "Bearing {} Compensation {} Output {}".format(int(message.heading), int(compensation), int(output))
        self.publishStatus()
        self.core.publish(MagnetoMessage(self.PROVIDE_BEARING_TOPIC, output))

    def processDistance(self, message: HeadInputMessage):
        self.lock.acquire()
        if message.servo in self.mapper:
            self.buffer[self.mapper[message.servo]] = message.ultrasonic
        self.lock.release()



class ObstacleAvoidanceNode2:
    PARAM_ZERO_ANGLE_OFFSET = "/obstacleAvoidance/zeroAngleOffset"
    PARAM_OBSTACLE_DISTANCE = "/obstacleAvoidance/obstacleDistance"
    PARAM_STOP_DISTANCE = "/obstacleAvoidance/stopDistance"
    PARAM_OBSTACLE_IGNORE_DISTANCE = "/obstacleAvoidance/obstacleIgnoreDistance"
    PARAM_OBSTACLE_OFFSET_DISTANCE = "/obstacleAvoidance/obstacleOffsetDistance"
    PARAM_OBSTACLE_OFFSET_COEF = "/obstacleAvoidance/obstacleOffsetCoef"
    PARAM_OBSTACLE_PASSED_OFFSET = "/obstacleAvoidance/obstaclePassedOffset"
    PARAM_SIDE_WAITING_OFFSET = "/obstacleAvoidance/obstacleSideWaitingOffset"
    PARAM_BEARING_CORRECTION_OVER_IGNORE_DISTANCE = "/obstacleAvoidance/BCOID"
    PROVIDE_STATUS_TOPIC = "/obstacleAvoidance/status"
    PROVIDE_BEARING_TOPIC = "/obstacleBearing"
    PROVIDE_OBSTACLE_DETECTED = "/obstacleAvoidance/detected"
    PROVIDE_OBSTACLE_PASSED = "/obstacleAvoidance/passed"
    REQUIRED_TICKS = "/control/engines/totalTicks"

    def __init__(self, core : Core):
        self.core = core
        self.alert = ValueMessage("/stop", "obstacle")
        self.core.registerTopic("/stop", ValueMessage)
        self.core.registerTopic(self.PROVIDE_BEARING_TOPIC, MagnetoMessage)
        self.core.registerTopic(self.PROVIDE_OBSTACLE_DETECTED, EmptyMessage)
        self.core.registerTopic(self.PROVIDE_OBSTACLE_PASSED, EmptyMessage)
        self.obstacleModeOffset = AngleDirection()
        self.obstacleMode = False
        self.obstacleWaitingSideDetection = False
        self.obstacleBearingOffset = AngleDirection()
        self.forwardAngles = [90, 50, 140]
        self.mapper = {
            0 : AngleDirection(self.core.getParam(self.PARAM_ZERO_ANGLE_OFFSET), Direction.Left),
            50 : AngleDirection(45, Direction.Left),
            90 : AngleDirection(90, Direction.Center),
            140 : AngleDirection(45, Direction.Right),
            180 : AngleDirection(self.core.getParam(self.PARAM_ZERO_ANGLE_OFFSET), Direction.Right)
        }
        self.core.registerTopic(self.PROVIDE_STATUS_TOPIC, ValueMessage)
        self.buffer = {}
        self.core.subscribe("/bearing", self.processBearing)
        self.core.subscribe("/himb", self.processDistance)
        self.core.registerTopic(self.PROVIDE_BEARING_TOPIC, MagnetoMessage)
        self.output = MagnetoMessage(self.PROVIDE_BEARING_TOPIC)
        self.lock = threading.Lock()
        self.core.subscribe(self.REQUIRED_TICKS, self.processTicks)
        self.leftTicks = ValueWithOffset(setPoint=self.core.getParam(self.PARAM_SIDE_WAITING_OFFSET))
        self.rightTicks = ValueWithOffset(setPoint=self.core.getParam(self.PARAM_SIDE_WAITING_OFFSET))

    def processTicks(self, message: ImprovedBridgeMessage):
        if self.obstacleWaitingSideDetection:
            if self.leftTicks.update(message.left) or self.rightTicks.update(message.right):
                self.obstacleWaitingSideDetection = False
        else:
            self.leftTicks.offset = message.left
            self.rightTicks.offset = message.right

    def publishStatus(self, status):
        # self.core.publish(ValueMessage(self.PROVIDE_STATUS_TOPIC, status))
        print(status)

    def processDistance(self, message: HeadInputMessage):
        self.lock.acquire()
        if message.servo in self.mapper:
            self.buffer[message.servo] = message.ultrasonic
        self.lock.release()

    def getDistanceByAngle(self, angle):
        distance = None
        self.lock.acquire()
        if angle in self.buffer:
            distance = self.buffer[angle]
        self.lock.release()
        # print("Dist by angle {} / {}".format(angle, distance))
        return distance

    def obstacleBalance(self, threshold, blacklist):
        if blacklist is None:
            blacklist = []
        directionProcessor = DirectionProcessor()
        self.lock.acquire()
        for angle in self.buffer:
            if angle in blacklist:
                continue
            if self.buffer[angle] < threshold:
                directionProcessor.append(self.mapper[angle])
        self.lock.release()
        return directionProcessor


    def checkForwardObstacle(self):
        for angle in self.forwardAngles:
            distance = self.getDistanceByAngle(angle)
            if distance is not None and distance < self.core.getParam(self.PARAM_OBSTACLE_DISTANCE):
                ad = self.mapper[angle]
                output = AngleDirection(ad.angle, ad.direction)
                if output.direction == Direction.Center:
                    balance = self.obstacleBalance(self.core.getParam(self.PARAM_OBSTACLE_DISTANCE), [])
                    output.direction = balance.directionBalance()
                    if output.direction == Direction.Center:
                        output.direction = Direction.Right
                    return output
                else:
                    return output
        return None

    def diffAngleValue(self, diffList):
        directionProcessor = DirectionProcessor()
        for diff in diffList:
            print("Diff {} {}".format(diff.direction, diff.angle))
            directionProcessor.append(diff)
        output = directionProcessor.directionAngleBalance().getAngleValue()
        print("Diff {}".format(output))
        return output

    def angleValueToAngleDirection(self, angle):
        if angle == 0:
            return AngleDirection(angle, Direction.Center)
        if angle > 0:
            return AngleDirection(angle, Direction.Right)
        else:
            return AngleDirection(abs(angle), Direction.Left)

    def getSideDistance(self):
        angle = None
        if self.obstacleBearingOffset.direction == Direction.Left:
            angle = 0
        else:
            angle = 180
        dist = self.getDistanceByAngle(angle)
        return dist

    def calcBearingCorection(self):
        distance = self.getSideDistance()
        if distance < self.core.getParam(self.PARAM_OBSTACLE_IGNORE_DISTANCE):
            offset = self.core.getParam(self.PARAM_OBSTACLE_OFFSET_DISTANCE)
            coef = self.core.getParam(self.PARAM_OBSTACLE_OFFSET_COEF)
            angle = (distance - offset) * coef
            return self.angleValueToAngleDirection(angle)
        else:
            return self.core.getParam(self.PARAM_BEARING_CORRECTION_OVER_IGNORE_DISTANCE)

    def processBearing(self, message: MagnetoMessage):
        print(self.buffer)
        diffList = []
        if self.obstacleMode:
            diffList.append(self.obstacleBearingOffset)
            if self.obstacleWaitingSideDetection:
                self.publishStatus("Waiting for obstacle side detection {}".format(self.getSideDistance()))
                if self.getSideDistance() < self.core.getParam(self.PARAM_OBSTACLE_IGNORE_DISTANCE):
                    self.obstacleWaitingSideDetection = False
            else:
                corection = self.calcBearingCorection()
                self.publishStatus("Passing obstacle {} {}".format(corection.direction, corection.angle))
                diffList.append(corection)
                dp = DirectionProcessor()
                dp.append(self.obstacleBearingOffset)
                dp.append(corection)
                originalDirection = self.obstacleBearingOffset
                self.obstacleBearingOffset = dp.directionAngleBalance()
                print("Original direction {}  obstacle bearing offset {}".format(originalDirection.direction, self.obstacleBearingOffset.direction))
                if self.obstacleBearingOffset.direction != originalDirection:
                    # self.core.publish(EmptyMessage(self.PROVIDE_OBSTACLE_PASSED))
                    self.publishStatus("Obstacle successfully passed")
                    self.obstacleBearingOffset = AngleDirection()
                    self.obstacleMode = False
        else:
            self.publishStatus("Waiting for obstacle")
            fo = self.checkForwardObstacle()
            self.publishStatus("Forward checking {}".format(fo))
            if fo is not None:
                # self.core.publish(EmptyMessage(self.PROVIDE_OBSTACLE_DETECTED))
                self.publishStatus("Forward Obstacle detected {} {}".format(fo.direction, fo.angle))
                self.obstacleMode = True
                self.obstacleWaitingSideDetection = True
                self.obstacleBearingOffset = fo
            self.publishStatus("Obstacle balancing")
            diffList.append(self.obstacleBalance(self.core.getParam(self.PARAM_OBSTACLE_DISTANCE), self.forwardAngles).directionAngleBalance())
            # self.publishStatus("After waiting for obstacle")

        # print("ObstacleNode2 calc heading")
        self.output.heading = AngleCalc.calcAngle(message.heading, self.diffAngleValue(diffList)*-1)
        print("ObstacleNode2 {} {}".format(self.output.topic, self.output))
        self.core.publish(self.output)


class ObstacleAvoidanceNode:
    REQUIRED_TOPIC = ["/him", "/heading"]
    PROVIDED_TOPIC = ["/obstacleHeading"]
    REQUIRED_PARAM = ["/obstacleAvoidance/obstacleDistance", "/obstacleAvoidance/stopDistance", "/obstacleAvoidance/zeroAngleOffset", "/obstacleAvoidance/forwardObstacleRange"]

    def __init__(self, core: Core):
        self.core = core
        self.buffer = {}
        self.core.subscribe("/heading", self.processHeading)
        self.core.subscribe("/him", self.processDistance)
        self.output = MagnetoMessage("/obstacleHeading")
        self.lock = threading.Lock()
        # self.obstacleDistance = ParamValue(core, "/obstacleAvoidance/obstacleDistance")
        # self.stopDistance = ParamValue(core, "/obstacleAvoidance/stopDistance")
        # self.zeroAngleOffset = ParamValue(core, "/obstacleAvoidance/zeroAngleOffset")
        # self.forwardObstacleRange = ParamValue(core, "/obstacleAvoidance/forwardObstacleRange")

    def processDistance(self, message: HeadInputMessage):
        self.lock.acquire()
        self.buffer[message.servo-90] = message.ultrasonic
        self.lock.release()

    def processHeading(self, message: MagnetoMessage):
        diff = 0
        forwardObstacle = False
        self.lock.acquire()
        for angle in self.buffer:
            distance = self.buffer[angle]
            # print("{} {}".format(angle,distance))
            if distance < self.core.getParam("/obstacleAvoidance/obstacleDistance"):
                if inRange(angle, 0, self.core.getParam("/obstacleAvoidance/forwardObstacleRange")):
                    forwardObstacle = True
                tmp = 90-abs(angle)
                if inRange(tmp, 0, 1):
                    if tmp > 0:
                        tmp = self.core.getParam("/obstacleAvoidance/zeroAngleOffset")
                    tmp = self.core.getParam("/obstacleAvoidance/zeroAngleOffset")
                if angle > 0:
                    tmp = tmp * -1
                diff += tmp
                print("{} {} {}".format( angle,diff, tmp))
        self.lock.release()
        if forwardObstacle and inRange(diff, 0, 70):
            diff = addSameSign(diff, (90-abs(diff)))
        self.output.heading = AngleCalc.calcAngle(message.heading, diff)
        self.core.publish(self.output)


if __name__ == '__main__':
    core = Core()
    himb = TestSubscriber(core, "/obstacleHeading", True)
    heading = MagnetoMessage()

    s = SerializationCoreBridge(core, "COM5", 115200)
    core.registerTopic(HeadInputMessage().topic, HeadInputMessage)
    s.subscribeArduino(HeadInputMessage().topic)
    s.start()
    
    node = ObstacleAvoidanceNode2(core)

    print("Ready to send")
    while True:
        core.publish(heading)
        time.sleep(0.25)

