from core.core import Core, Node
from messages.GpsMessage import GpsMessage
from messages.core.Message import ValueMessage
from nav.calc.Stat import Stat
from nav.calc.Buffers import Buffer


class BufferedGpsNode(Node):
    """
    require /moving = messages.core.Message.ValueMessage [True/False]
    """

    def __init__(self, core: Core, gpsTopic="/gps", pubTopic="/bgps", latOffset=0, lonOffset=0):
        super().__init__(core)
        self.latBuffer = Buffer()
        self.lonBuffer = Buffer()
        self.latOffset = latOffset
        self.lonOffset = lonOffset
        self.pubTopic = pubTopic
        self.core.subscribe(gpsTopic, self.gpsIn)
        self.core.subscribe("/moving", self.moving)
        self.buffering = False
        self._valid = False

    def winMean(self, data):
        return Stat.mean(Stat.winsorize(data, 0.2))

    def gpsIn(self, message: GpsMessage):
        if message.valid and message.fix > 1 and self.buffering:
            self.latBuffer.append(message.lat)
            self.lonBuffer.append(message.lon)
            message.lat = self.winMean(self.latBuffer)+self.latOffset
            message.lon = self.winMean(self.lonBuffer)+self.lonOffset

        message.topic = self.pubTopic
        self.core.publish(message)

    def moving(self, message: ValueMessage):
        self.buffering = not message.value
        if message.value:
            self.latBuffer.clear()
            self.lonBuffer.clear()



