from core.core import Core
from messages.EncoderMessage import EncoderMessage
from messages.core.Message import ValueMessage


class EncoderInfoNode:
    """
    publishing /moving
    subscribing EncoderMessage
    """
    MOVE_FORWARD = 1
    MOVE_STOP = 0
    MOVE_BACKWARD = -1

    def __init__(self, core: Core, topic="", timePerUpdate=100, ticksPerRevolution=180, wheelDiameter=0, move=MOVE_STOP, callback=None):
        self.core = core
        self.callback = callback
        self.wheelDiameter = wheelDiameter
        self.timePerUpdate = timePerUpdate
        self.ticksPerRevolution = ticksPerRevolution
        self.move = move
        self.totalTicks = 0
        self.tempTicks = 32
        self.lastTicks = 0
        self.speed = 0
        self.tempSpeed = 0
        self.movingMsg = ValueMessage("/moving", False)
        core.subscribe(topic, self.publish)

    def publish(self, message: EncoderMessage):
        self.move = message.move
        self.moving()
        self.calc(message.move, message.ticks)
        if self.callback is not None:
            self.callback(self)

    def moving(self):
        if self.move == self.MOVE_STOP:
            self.movingMsg.value = False
        else:
            self.movingMsg.value = True
        self.core.publish(self.movingMsg)

    def calc(self, move, ticks):
        if move == self.MOVE_FORWARD:
            self.lastTicks = ticks
            self.tempTicks += ticks
            self.totalTicks += ticks
        if move == self.MOVE_BACKWARD:
            self.lastTicks = ticks * -1
            self.tempTicks -= ticks
            self.totalTicks += ticks

    def calcSpeed(self, ticks):
        pass
