import queue
from utils.QueueUtils import addAll
import enum
from messages.core.Message import Message


class Direction(enum.Enum):
    Left = -1
    Center = 0
    Right = 1


class Point(Message):

    def __init__(self, topic="/point", x=0, y=0, **kwargs):
        super().__init__(topic)
        self.x = x
        self.y = y


class GeoPoint(Message):

    def __init__(self, topic="/geopoint", lat=0, lon=0, **kwargs):
        super().__init__(topic)
        self.lat = lat
        self.lon = lon

    def __str__(self):
        return "Lat: {}\tLon: {}".format(self.lat, self.lon)


class WayPoint(GeoPoint):

    def __init__(self, topic="/waypoint", name="", lat=0, lon=0, **kwargs):
        super().__init__(topic, lat, lon)
        self.name = name

    def __str__(self):
        return "{} {}".format(self.name, super().__str__())


class Path(Message):

    def __init__(self, topic="/path", name="", waypoints=None, **kwargs):
        super().__init__(topic)
        self.name = name
        self.waypoints = waypoints or []

    def addWaypoint(self, waypoint: WayPoint):
        self.waypoints.append(waypoint)


if __name__ == '__main__':
    wps = [WayPoint("WP1"), WayPoint("WP2"), WayPoint("WP3")]
    q = queue.Queue()
    addAll(q, wps)
    while q.qsize() > 0:
        print(q.qsize())
        print(q.get())
    print(q)
    print(wps)