import time

import jsonpickle
import serial
import pynmea2

from core.core import Core, TestSubscriber
from messages.GpsMessage import GpsMessage
import threading

from messages.core.Message import ValueMessage, EmptyMessage
from nav.NavObjects import Path
import utils.IOUtils
from nodes.GpsPathOperationNode import PathsWrapper
from socketio.socketio import FlaskBridge


class GpsParserNode:
    PUBLISH_GPS_TOPIC = "/gps"

    def __init__(self, core: Core):
        self.core = core
        self.core.registerTopic(self.PUBLISH_GPS_TOPIC, GpsMessage)
        self.output = GpsMessage()
        self.conn = serial.Serial(timeout=0.1, port=core.getParam("/arduino/gps/com"), baudrate=core.getParam("/arduino/gps/baud"))
        self.reader = pynmea2.NMEAStreamReader()
        self.alive = True
        t = threading.Thread(target=self.process)
        t.start()

    def process(self):
        while self.alive:
            data = self.conn.read(1)
            data = data.decode("utf-8")
            try:
                for msg in self.reader.next(data):
                    self.output.fix = msg.gps_qual
                    self.output.sat = msg.num_sats
                    self.output.lat = msg.latitude
                    self.output.lon = msg.longitude
                    self.core.publish(self.output)
            except:
                pass

class FakeGpsNode:
    PUBLISH_GPS_TOPIC = "/gps"
    REQIRE_PROCESS_FAKE_GPS = "/fakeGps"
    def __init__(self, core: Core):
        self.core = core
        self.core.registerTopic(self.PUBLISH_GPS_TOPIC, GpsMessage)
        pw = jsonpickle.decode(utils.IOUtils.readUtf8("paths.json"))
        self.paths = pw.paths
        self.core.registerTopic(self.REQIRE_PROCESS_FAKE_GPS, ValueMessage)
        self.core.subscribe(self.REQIRE_PROCESS_FAKE_GPS, self.processGps)
        self.core.subscribe("/nav/onPoint", self.processOnPoint)
        self.core.subscribe("/taskProcessor/complete", self.tasksComplete)
        self.waitingForTasks = False
        self.t = None

    def processGps(self, message: ValueMessage):
        path: Path = None
        if message.value in self.paths:
            path = self.paths[message.value]
        if path is not None:
            self.t = threading.Thread(target=self.fakeGps, args=(path, ))
            self.t.start()

    def fakeGps(self, path: Path):
        for waypoint in path.waypoints:
            while self.waitingForTasks:
                time.sleep(1)
            self.core.publish(GpsMessage(lat=waypoint.lat, lon=waypoint.lon))
            time.sleep(1)

    def processOnPoint(self, message: EmptyMessage):
        self.waitingForTasks = True

    def tasksComplete(self, message: EmptyMessage):
        self.waitingForTasks = False


def read(filename):
    f = open(filename)
    reader = pynmea2.NMEAStreamReader(f)

    while 1:
        for msg in reader.next():
          print(msg)


def read_serial(filename):
    com = None
    reader = pynmea2.NMEAStreamReader()

    while 1:

        if com is None:
            try:
                com = serial.Serial(timeout=5.0, port="COM10", baudrate=9600)
            except serial.SerialException:
                print('could not connect to %s' % filename)
            time.sleep(5.0)
            continue

        data = com.read(1)
        data = data.decode("utf-8")
        try:
            for msg in reader.next(data):
                print(msg.num_sats)
                # print(msg)
        except:
            pass
            # print("ERR")

def x():
    con = serial.Serial(port="COM10", baudrate=9600)
    while True:
        msg = pynmea2.parse(chr(con.read()[0]))
        print(msg.num_sats)


if __name__ == '__main__':
    core = Core()
    gps = GpsParserNode(core)
    ts = TestSubscriber(core, "/gps", True)