from core.core import Core
from serialization.SerializationData import SerializationData
from serialization.SerializationSerialConnection import SerializationSerialConnection


class SerializationCoreBridge:

    def __init__(self, core: Core, port="COM8", baud=115200):
        self.core = core
        self.conn = SerializationSerialConnection(port, baud)
        self.topicArduino = {}

    def start(self):
        self.conn.start()

    def subscribeArduino(self, topic, preamble=None, postamble=None):
        msg = self.core.getTopicClass(topic)()
        msg.topic = topic
        if preamble is not None:
            msg.preamble = preamble
        if postamble is not None:
            msg.postamble = postamble
        msg.addListener(self)
        self.conn.addDeserializableData(msg)

    def subscribeCore(self, topic, preamble, postamble):
        self.topicArduino[topic] = [preamble, postamble]
        self.core.subscribe(topic, self.write);

    def dataUpdate(self, data):
        #print("Data Updated {}".format(str(data)))
        self.core.publish(data)

    def write(self, message: SerializationData):
        if message.topic in self.topicArduino:
            list = self.topicArduino[message.topic]
            message.preamble = list[0]
            message.postamble = list[1]
        #print("To Arduino: {}".format(message))
        self.conn.write(message)

