from serialization.SerializationSerialReader import SerializationSerialReader
from serialization.SerializationOutputStream import SerializationOutputStream
from serialization.SerializationData import SerializationData
import threading
import serial
import time


class SerializationSerialConnection(threading.Thread):
    SERIAL_UPDATE_DELAY = 0.01
    ser: serial.Serial
    readers = []
    alive = True

    def __init__(self, port, baud):
        self.ser = serial.Serial(port= port, baudrate = baud, timeout=1)
        threading.Thread.__init__(self)

    def addDeserializableData(self, data: SerializationData):
        reader = SerializationSerialReader(data)
        self.addReader(reader)

    def addReader(self, reader):
        self.readers.append(reader)

    def run(self) -> None:
        print("Serial run")
        while self.alive:
            #while self.ser.in_waiting > 0:
            b = self.ser.read(1)
            if len(b) == 0:
                continue
            b = ord(b)
            #print(b)
            for reader in self.readers:
                reader.processData(b)
        time.sleep(self.SERIAL_UPDATE_DELAY)

    def write(self, data: SerializationData):
        output = SerializationOutputStream(data.getSize())
        data.write(output)
        self.ser.write(output.buffer)
        #print("OUT: "+str(len(output.buffer)))
        #print(output.buffer)
        self.ser.flush()

    def close(self):
        self.alive = False
        time.sleep(1)
        self.ser.close()
