from serialization.SerializationTypes import SerializationTypes
from serialization.SerializationData import SerializationData
from serialization.SerializationInputStream import SerializationInputStream
from ArrayUtils import ArrayUtils

class SerializationSerialReader:
    BUFFER_SIZE = 2048
    data: SerializationData = None
    buffer = []
    bufferPointer = 0
    doCollectData = False
    preamble = []
    postamble = []
    preamblePointer = 0
    postamblePointer = 0

    def __init__(self, data: SerializationData):
        self.buffer = ArrayUtils.initSize(self.buffer, self.BUFFER_SIZE)
        self.data = data
        #self.preamble = ArrayUtils.initSize(self.preamble, SerializationTypes.SIZEOF_INTEGER)
        #self.postamble = ArrayUtils.initSize(self.postamble, SerializationTypes.SIZEOF_INTEGER)
        self.preamble = SerializationTypes.integerToBytes(data.preamble)
        self.postamble = SerializationTypes.integerToBytes(data.postamble)
        print("preamble size "+str(len(self.preamble)))
        print(str(self.preamble[0])+" "+str(self.preamble[1])+" "+str(self.preamble[2])+" "+str(self.preamble[3]))

    def processData(self, b):
        if self.doCollectData:
            self.collectData(b)
        else:
            if b == self.preamble[self.preamblePointer]:
                self.buffer[self.preamblePointer] = b
                self.preamblePointer += 1
            else:
                self.preamblePointer = 0

            if self.preamblePointer == SerializationTypes.SIZEOF_INTEGER:
                self.preamblePointer = 0
                self.bufferPointer = SerializationTypes.SIZEOF_INTEGER - 1
                self.doCollectData = True

    def collectData(self, b):
        if self.bufferPointer < self.BUFFER_SIZE:
            self.bufferPointer += 1

        self.buffer[self.bufferPointer] = b

        if len(self.postamble) > self.postamblePointer:
            if b != self.postamble[self.postamblePointer]:
                self.postamblePointer = 0
            else:
                self.postamblePointer += 1
        else:
            self.postamblePointer = 0
            self.doCollectData = False

        if self.postamblePointer == SerializationTypes.SIZEOF_INTEGER:
            checksum = []
            checksum = ArrayUtils.initSize(checksum, SerializationTypes.SIZEOF_INTEGER)
            for i in range(SerializationTypes.SIZEOF_INTEGER):
                checksum[i] = self.buffer[i+SerializationTypes.SIZEOF_INTEGER]

            if SerializationTypes.bytesToInteger(checksum) == self.bufferPointer + 1:
                input = SerializationInputStream(self.buffer)
                self.data.read(input)
                #print(self.buffer)
                self.data.notifyListeners()
            else:
                pass
                # print("SerializationReader: Checksum failed")

            self.postamblePointer = 0
            self.doCollectData = False

