from socketIO_client import SocketIO, BaseNamespace, SocketIONamespace
import time
from threading import Thread
from core.mapper import mapper
import json
import queue as queue
from messages.arduino.KvMessage import KvMessage, MessageModeType
from HeadInputMessage import HeadInputMessage
from HeadInputMessageListener import HeadMessageListener
from core.core import Core
import utils.JsonUtils

JSON_MODE_PICKLE = 1

class Namespace(SocketIONamespace):

    def addCore(self, core):
        self.core = core

    def on_connect(self):
        print('[Connected]')

    def on_reconnect(self):
        print('[Reconnected]')

    def on_disconnect(self):
        print('[Disconnected]')

    def on_message(self, data):
        print(data)

    #def on_event(self, event, *args):
    def on_serverBridge(self, event, *args):
        #print(args)
        #print(event)
        tmp = event["msg"]
        #print(args[0]["msg"])
        #print(utils.JsonUtils.jsonToObject(tmp))
        #self.core.publish(utils.JsonUtils.jsonToObject(tmp))
        x = json.loads(tmp)
        #print(x)
        clazz = self.core.getTopicClass(x["topic"])
        if clazz is not None:
            jsonMode = self.core.getTopicKwargs(x["topic"])
            if jsonMode is not None and"json" in jsonMode and jsonMode["json"] == JSON_MODE_PICKLE:
                self.core.publish(utils.JsonUtils.toObjectPickle(tmp, clazz))
            else:
                self.core.publish(utils.JsonUtils.toObject(tmp, clazz))
        #if x["topic"] in mapper.keys():
        #    self.core.publish(utils.JsonUtils.toObject(tmp, mapper[x["topic"]]))
        # if self.core.getTopicClass(x["topic"]) != None:
        #     self.core.publish(utils.JsonUtils.toObject(tmp, self.core.getTopicClass([x["topic"]])))
        # else:
        #     raise ValueError("Core nemá přiřazenou obsluhu pro topic: {}".format(x["topic"]))


class FlaskBridge:

    def __init__(self, core: Core):
        self.core = core
        #self.socketIO = SocketIO('127.0.0.1', 5000)

        self.socketIO = SocketIO(core.getParam("/socketio/ip"), core.getParam("/socketio/port"))
        self.q = queue.Queue()
        ns = self.robotNamespace = self.socketIO.define(Namespace, '/pubsub')
        ns.addCore(core)
        #self.socketIO.on('event', self.on_event, '/pubsub')
        #self.socketIO.on('message', self.on_message, '/pubsub')
        #self.socketIO.on('connect', FlaskBridge.on_connect, '/pubsub')
        self.receive_thread = Thread(target=self.socketIO.wait)
        self.receive_thread.daemon = False
        self.receive_thread.start()
        #core.subscribe("/test", self)
        #core.subscribe("/him", self)
        self.t = Thread(target=self.sender)
        self.t.daemon = False
        self.t.start()

    def subscribeTopic(self, topic):
        self.core.subscribe(topic, self.publish)

    def publish(self, message):
        #print(message)
        #self.robotNamespace.emit('robotIn', {'msg': message})
        #print(type(message))
        #print(str(utils.JsonUtils.toJson(message)))
        self.q.put(message)
        #self.robotNamespace.emit("bridgeServer", {'msg': str(utils.JsonUtils.toJson2(message))})
        #self.robotNamespace.emit(message.topic, {'msg': "Ahoj"})
        # m = HeadInputMessage()
        # m.lidar = 32
        # m.lidarSignal = 64
        # m.ultrasonic = 128
        # m.irCode = 3
        #
        # m.addListener(HeadMessageListener())
        # self.robotNamespace.emit(message.topic, {'msg': str(utils.JsonUtils.toJson(m))})

    def sender(self):
        while True:
            if not self.q.empty():
                #print("QS_SIO "+str(self.q.qsize()))
                output = self.q.get()
                jsonMode = self.core.getTopicKwargs(output.topic)
                if jsonMode is not None and "json" in jsonMode and jsonMode["json"] == JSON_MODE_PICKLE:
                    self.robotNamespace.emit("bridgeServer", {'msg': str(utils.JsonUtils.toJson2Pickle(output))})
                else:
                    self.robotNamespace.emit("bridgeServer", {'msg': str(utils.JsonUtils.toJson2(output))})
            else:
                time.sleep(0.01)


    def on_event(self, event):
        print(event)
        #print(args)

    def on_message(self, message):
        print(message)

    @staticmethod
    def on_connect():
        print('connect')

    def close(self):
        self.socketIO.disconnect()


class FlaskBridgeParamNode:
    CORE_BRIDGE_TOPIC = "/coreBridgeParam"
    BRIDGE_CORE_TOPIC = "/bridgeCoreParam"
    REFRESH_TIMEOUT = 1

    def __init__(self, core: Core, fb: FlaskBridge, topics):
        self.core = core
        self.fb = fb
        self.core.subscribe(self.BRIDGE_CORE_TOPIC, self.process)
        self.topics = topics
        for topic in topics:
            temp = self.BRIDGE_CORE_TOPIC+topic
            self.core.registerTopic(temp, KvMessage)
            self.core.subscribe(temp, self.process)
            self.fb.subscribeTopic(self.CORE_BRIDGE_TOPIC+topic)
        self.topicOffset = len(self.CORE_BRIDGE_TOPIC)
        self.alive = True
        self.t = Thread(target=self.run)
        self.t.daemon = False
        self.t.start()

    def process(self, message: KvMessage):
        if message.value is not None:
            self.core.setParam(message.topic[self.topicOffset:], message.value)
            #if len(message.value) > 0:
            #    self.core.setParam(message.topic[self.topicOffset:], message.value)

    def stop(self):
        self.alive = False

    def run(self):
        while self.alive:
            for topic in self.topics:
                m = KvMessage(self.CORE_BRIDGE_TOPIC+topic, value=self.core.getParam(topic))
                self.core.publish(m)
            time.sleep(self.REFRESH_TIMEOUT)











#socketIO = SocketIO('127.0.0.1', 5000)
#chatNamespace = socketIO.define(Namespace, '/pubsub')
#socketIO = SocketIO('127.0.0.1', 5000, Namespace)
#chatNamespace.emit('robotIn', {'msg': "Ahoooooj z robota"})
#socketIO.emit('robotIn', {'msg': "Ahoooooj z robota"})
#socketIO.wait()


if __name__ == '__main__':
    core = Core()
    fb = FlaskBridge(core)
    print("Started")
    for x in range(3):
        fb.publish("Ahoj z mostu")
        time.sleep(1)
    fb.close()
    print("Closed")