from core.core import Core, X
from socketio.socketio import FlaskBridge
from HeadOutputMessage import HeadOutputMessage
from HeadInputMessage import HeadInputMessage
from UltrasonicParamsMessage import UltrasonicParamsMessage
from serialization.SerializationSerialConnection import SerializationSerialConnection
from messages.arduino.BridgeMessage import BridgeMessage
from keyboard.keyboard import HidInputPublisher


class CoreSender:

    def __init__(self, core: Core):
        self.core = core

    def dataUpdate(self, data):
        self.core.publish(data)

def bridge(left, right, message: BridgeMessage):
    if left > 0 and right > 0:
        message.mode = 0
    if left < 0 and right < 0:
        message.mode = 1
    if left > 0 and right < 0:
        message.mode = 2
    if left < 0 and right > 0:
        message.mode = 3
    message.left = abs(left)
    message.right = abs(right)

class MotorDriver:
    def __init__(self, core: Core):
        listener = CoreSender(core)
        self.conn = SerializationSerialConnection("COM3", 115200)
        self.conn.start()

    def publish(self, message):
        #t1 = getCurrentTimeMillis()
        bm = BridgeMessage()
        bm.preamble = 4294900001
        bm.postamble = 4294900002
        bridge(int(message.left), int(message.right), bm)
        self.conn.write(bm)
        #t2 = getCurrentTimeMillis()
        #print("MD "+str(t2-t1))



class AP:

    def __init__(self, core: Core):
        listener = CoreSender(core)
        #self.message = HeadInputMessage()
        self.hom = HeadInputMessage()
        self.hom.addListener(listener)
        self.conn = SerializationSerialConnection("COM4", 115200)
        self.conn.addDeserializableData(self.hom)
        #self.conn.addDeserializableData(self.message)
        self.conn.start()

    def publish(self, message):
        #print(int(message.servo))
        #print(message.servo)
        #t1 = getCurrentTimeMillis()
        h = HeadOutputMessage()
        h.preamble = 4294900001
        h.postamble = 4294900002
        h.angle = int(message.servo)
        h.mode = 2
        h.delay = 100
        h.rate = 100
        self.conn.write(h)
        m = UltrasonicParamsMessage()
        m.delay = 100
        self.conn.write(m)
        #t2 = getCurrentTimeMillis()
        #print("AP "+str(t2-t1))
        # bm = BridgeMessage()
        # bm.preamble = 4294900001
        # bm.postamble = 4294900002
        # bridge(int(message.left), int(message.right), bm)
        # self.conn.write(bm)

        # h = HeadOutputMessage()
        # h.preamble = 4294900001
        # h.postamble = 4294900002
        # h.angle = 160
        # self.conn.write(h)
        # m = UltrasonicParamsMessage()
        # m.delay = 80
        # print(m.delay)
        # self.conn.write(m)
        # time.sleep(1)
        # h.angle = 20
        # self.conn.write(h)
        # m.delay = 50
        # print(m.delay)
        # self.conn.write(m)
        # time.sleep(1)

core = Core()
fb = FlaskBridge(core)
x = X()
md = MotorDriver(core)
ap = AP(core)
core.subscribeAll(x)
hid = HidInputPublisher(core, "/hid")
core.subscribe("/test", ap)
core.subscribe("/test", md)
