from keyboard.keyboard import HidInputPublisher
from messages.arduino.Imu import Imu
from serialization.SerializationCoreBridge import SerializationCoreBridge
from messages.arduino.KvMessage import KvMessage, KvIntMessage, KvByteMessage, KvFloatMessage
from core.core import Core, TestSubscriber, TestPublisher
import time
from nav.ImuTransformationNode import ImuTransform
from socketio.socketio import FlaskBridge
from messages.TestMessage import TestMessage
from nav.EnginesController import EnginesController, SpeedMode, DirectionMode
from engines.DifferentialEnginesControl import DifferentialEnginesControl
from messages.MagnetoMessage import MagnetoMessage
from messages.core.Message import ValueMessage
from messages.socket.SocketMessages import StringMessage
import nav.calc.Calc
from simple_pid import PID


if __name__ == '__main__':
    core = Core()
    hid = HidInputPublisher(core, "/hid")
    core.registerTopic(Imu().topic, Imu)
    core.registerTopic("/test", TestMessage)
    core.registerTopic(KvByteMessage().topic, KvByteMessage)

    s = SerializationCoreBridge(core, "COM8", 115200)
    s.subscribeArduino(Imu().topic)
    s.subscribeCore(KvByteMessage().topic, 4294901763, 4294901764)
    s.subscribeCore("/imu/magnetometer/calibrate", 4294901763, 4294901764)
    s.subscribeCore(KvIntMessage().topic, 4294901765, 4294901766)
    s.start()

    s2 = SerializationCoreBridge(core, "COM10", 115200)
    s2.subscribeCore("/control/engines/output", 123456789, 987654321)
    s2.start()

    tr = ImuTransform(core)

    time.sleep(10)
    msg = KvIntMessage()
    msg.value = 100
    core.publish(msg)

    fb = FlaskBridge(core)
    fb.subscribeTopic("/imu")

    ec = EnginesController(core)
    dif = DifferentialEnginesControl(core)
    #fc = FollowCards(core, ec)


class FollowCards:

    def __init__(self, core: Core, ec: EnginesController):
        self.core = core
        self.ec = ec
        #self.core.subscribe("/heading", self.inputHeading)
        ec.speedController.setMode(SpeedMode.GAMEPAD_INPUT,90)
        self.core.subscribe("/hid", self.inputHid)
        self.core.subscribe("/str", self.inputHid)
        #ec.directionController.setMode(DirectionMode.GAMEPAD_INPUT, 90)
        self.speed = core.getParam("/followCards/speed")

        self.bearing = core.getParam("/followCards/angle1")
        self.lastHeading = 0

    def inputHid(self, message: StringMessage):
        if self.bearing == self.core.getParam("/followCards/angle2"):
            self.bearing = self.core.getParam("/followCards/angle1")
        else:
            self.bearing = self.core.getParam("/followCards/angle2")
        self.core.publish(ValueMessage("/followCards/bearing", self.bearing))
        self.ec.directionController.setMode(DirectionMode.HOLD_ANGLE, self.bearing)

    def inputHeading(self, message: MagnetoMessage):
        if nav.calc.Calc.inRange(message.heading, self.bearing, self.core.getParam("/followCards/okAngle")):
            self.core.publish(ValueMessage("/followCards/speed", self.speed))
            self.ec.speedController.setMode(SpeedMode.HOLD_SPEED, self.speed)
        else:
            self.core.publish(ValueMessage("/followCards/speed", 90))
            self.ec.speedController.setMode(SpeedMode.HOLD_SPEED, 90)


class ManualControl:

    def __init__(self, core: Core, ec: EnginesController):
        self.core = core
        self.ec = ec
        ec.speedController.setMode(SpeedMode.GAMEPAD_INPUT, 90)
        ec.directionController.setMode(DirectionMode.GAMEPAD_INPUT, 90)


